__authors__ = ["Tuxa"]
__date__ = "20201102"
__version__ = "0.1a4"
__appname__ = "glxsh"
__description__ = "Galaxie Shell"
__licence__ = """Copyright (C) 2020 Galaxie Shell Project.
License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>.
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.
"""
